// ---------------------------------------------------------------
// Copyright 2004, SAS Institute Inc.
// ---------------------------------------------------------------
using System;
using System.Windows.Forms;
using System.Xml;		// for XMLTextWriter and XMLTextReader
using System.IO;		// for StreamReader and StreamWriter

using SAS.Shared.AddIns;	// for access to the Add-in interfaces

namespace EGAddin.CharacterizeData
{
	/// <summary>
	/// Characterize Data - an add-in for Enterprise Guide and the SAS Add-in for Microsoft Office.
	/// 
	/// </summary>
	public class CharacterizeData : ISASTask, ISASTaskDescription, ISASTaskAddIn
	{
		#region private members 

		// consumer object supplied by host application
		private ISASTaskConsumer consumer = null;

		// member variables to store state information
		private string library = "WORK";
		private bool bCharts = false;
		private bool bAllMembers = true;
		private string dataset = "";
		private int catobs = 30;

		#endregion

		/// <summary>
		/// Default constructor
		/// </summary>
		public CharacterizeData()
		{
		}

		#region ISASTaskAddIn Members

		public bool VisibleInManager
		{
			get
			{
				return true;
			}
		}

		public string AddInName
		{
			get
			{
				return "Characterize Data";
			}
		}

		public void Disconnect()
		{
			consumer = null;
		}

		public int Languages(out string[] Items)
		{
			Items = new string[] { "en-US" };
			return 1;
		}

		public string AddInDescription
		{
			get
			{
				return "Analyze data in a library and report on its characteristics.";
			}
		}

		public bool Connect(ISASTaskConsumer Consumer)
		{
			consumer = Consumer;

			return true;
		}

		public string Language
		{
			set
			{
				// we support english only
			}
		}

		#endregion

		#region ISASTaskDescription Members

		public string ProductsRequired
		{
			get
			{
				return "BASE";
			}
		}

		public bool StandardCategory
		{
			get
			{
				return false;
			}
		}

		public bool GeneratesListOutput
		{
			get
			{
				return true;
			}
		}

		public string TaskName
		{
			get
			{
				return "Characterize Data";
			}
		}

		public string Validation
		{
			get
			{
				return "SAS Sample";
			}
		}

		public string TaskCategory
		{
			get
			{
				return Global.Category;
			}
		}

		public string IconAssembly
		{
			get
			{
				// return the full path/name of this assembly
				return System.Reflection.Assembly.GetExecutingAssembly().Location;
			}
		}

		public string Clsid
		{
			get
			{
				return "6367fa75-6730-40d7-8d8b-3744baf252ad";
			}
		}

		public SAS.Shared.AddIns.ShowType TaskType
		{
			get
			{
				return SAS.Shared.AddIns.ShowType.Wizard;
			}
		}

		public bool RequiresData
		{
			get
			{
				return false;
			}
		}

		public string IconName
		{
			get
			{
				// return the name of the icon within our assembly
				// including namespace qualifiers
				return "EGAddin.CharacterizeData.Characterize.ico";
			}
		}

		public int MinorVersion
		{
			get
			{
				return 0;
			}
		}

		public int MajorVersion
		{
			get
			{
				return 1;
			}
		}

		public int NumericColsRequired
		{
			get
			{
				return 0;
			}
		}

		public string TaskDescription
		{
			get
			{
				return "Characterize the data in a library";
			}
		}

		public bool GeneratesSasCode
		{
			get
			{
				return true;
			}
		}

		public string FriendlyName
		{
			get
			{
				return "Characterize Data";
			}
		}

		public string ProcsUsed
		{
			get
			{
				return "DATA step; SUMMARY";
			}
		}

		public string WhatIsDescription
		{
			get
			{
				return "Add some stuff here";
			}
		}

		public string ProductsOptional
		{
			get
			{
				return "";
			}
		}

		#endregion

		#region ISASTask Members

		public string RunLog
		{
			get
			{
				return null;
			}
		}

		public string XmlState
		{
			get
			{
				return WriteXML();
			}
			set
			{
				ReadXML(value);
			}
		}

		public void Terminate()
		{
			// TODO:  cleanup for this run
		}

		public SAS.Shared.AddIns.OutputData OutputDataInfo(int Index, out string Source, out string Label)
		{
			Source = null;
			Label = null;
			return new SAS.Shared.AddIns.OutputData();
		}

		public string SasCode
		{
			get
			{
				string member = (bAllMembers || dataset.Length==0) ? "_ALL_" : dataset;
				string co = catobs.ToString();
				string macro = Global.ReadFileFromAssembly("EGAddin.CharacterizeData.Characterize_Data.sas");
				string call = string.Format("%charact(dsn={0},lib={1},catobs={2});",member,library, co);
				string charts = (bCharts) ? Global.ReadFileFromAssembly("EGAddin.CharacterizeData.GraphCode.sas") : "";

				string code = macro + Environment.NewLine + call + Environment.NewLine + charts;

				return code;
			}
		}

		public bool Initialize()
		{
			// TODO:  init
			return true;
		}

		public SAS.Shared.AddIns.ShowResult Show(System.Windows.Forms.IWin32Window Owner)
		{
			// launch UI

			// create the dialog class
			CharacterizeDataForm dlg = new CharacterizeDataForm();

			// set the consumer interface so the dialog can call back into the
			// application
			dlg.Consumer = consumer;
			dlg.Library = library;
			dlg.IncludeCharts = bCharts;
			dlg.DataSet = dataset;
			dlg.AllMembers = bAllMembers;
			dlg.Catobs = catobs;

			if (DialogResult.OK == dlg.ShowDialog())
			{
				library = dlg.Library;
				bCharts = dlg.IncludeCharts;
				bAllMembers = dlg.AllMembers;
				catobs = dlg.Catobs;
				if (!bAllMembers)
					dataset = dlg.DataSet;

				return SAS.Shared.AddIns.ShowResult.RunNow;
			}
			else
				return SAS.Shared.AddIns.ShowResult.Canceled;
		}

		public string Label
		{
			get
			{
				return "Characterize Data";
			}
			set
			{
				// TODO:  Add CharacterizeData.Label setter implementation
			}
		}

		public int OutputDataCount
		{
			get
			{		
				return 0;
			}
		}

		#endregion

		#region Save and load XML state
		private string WriteXML()
		{
			StringWriter sw = new StringWriter();

			XmlTextWriter writer = new XmlTextWriter(sw);
			writer.WriteStartElement("CharacterizeData");
			writer.WriteElementString("Library", library);
			writer.WriteElementString("DataSet", dataset);
			writer.WriteElementString("ShowCharts",XmlConvert.ToString(bCharts));
			writer.WriteElementString("AllMembers",XmlConvert.ToString(bAllMembers));
			writer.WriteElementString("CatObs",XmlConvert.ToString(catobs));
			writer.WriteEndElement();
			writer.Close();

			return sw.ToString();
		}

		private void ReadXML(string xml)
		{
			if (xml!=null && xml.Length>0)
			{
				try
				{
					StringReader sr = new StringReader(xml);
					XmlTextReader reader = new XmlTextReader(sr);
					reader.ReadStartElement("CharacterizeData");
					library = reader.ReadElementString("Library");
					dataset = reader.ReadElementString("DataSet");
					bCharts = XmlConvert.ToBoolean(reader.ReadElementString("ShowCharts"));
					bAllMembers = XmlConvert.ToBoolean(reader.ReadElementString("AllMembers"));
					catobs = XmlConvert.ToInt32(reader.ReadElementString("CatObs"));
					reader.ReadEndElement();
					reader.Close();
				}
				catch
				{
				}
			}
		}
		#endregion


	}
}
