// ---------------------------------------------------------------
// Copyright 2004, SAS Institute Inc.
// ---------------------------------------------------------------
using System;
using System.Windows.Forms;
using System.Xml;		// for XMLTextWriter and XMLTextReader
using System.IO;		// for StreamReader and StreamWriter
using System.Text;		// for StringBuilder

using SAS.Shared.AddIns;	// for access to the Add-in interfaces


namespace EGAddin.Contents
{
	/// <summary>
	/// Summary description for Contents.
	/// </summary>
	public class Contents : ISASTask, ISASTaskAddIn, ISASTaskDescription
	{
		public enum OutputFormat : int
		{
			Report,
			Data,
			ReportAndData
		}

		#region private members 

		// consumer object supplied by host application
		private ISASTaskConsumer consumer = null;
		private string label = "Data Set Contents";

		private OutputFormat output = OutputFormat.Report;
		private bool details = true;
		private bool ignorecase = false;

		#endregion

		public Contents()
		{
		}

		#region ISASTaskAddIn Members

		public bool VisibleInManager
		{
			get
			{
				return true;
			}
		}

		public string AddInName
		{
			get
			{
				return "Data Set Contents";
			}
		}

		public void Disconnect()
		{
			consumer = null;
		}

		public int Languages(out string[] Items)
		{
			Items = new string[] { "en-US" };
			return 1;
		}

		public string AddInDescription
		{
			get
			{
				return "Run PROC CONTENTS to create a report about the input data.";
			}
		}

		public bool Connect(ISASTaskConsumer Consumer)
		{
			consumer = Consumer;
			if (consumer==null || consumer.ActiveData==null)
				throw new AddInException("Cannot get to active data",ErrorType.None);

			label = string.Format("Data set contents for {0}.{1}",consumer.ActiveData.Library, consumer.ActiveData.Member);
			
			return true;
		}

		public string Language
		{
			set
			{
				// no implementation
			}
		}

		#endregion

		#region ISASTaskDescription Members

		public string ProductsRequired
		{
			get
			{
				return "BASE";
			}
		}

		public bool StandardCategory
		{
			get
			{
				return false;
			}
		}

		public bool GeneratesListOutput
		{
			get
			{
				return true;
			}
		}

		public string TaskName
		{
			get
			{
				return "Data set contents";
			}
		}

		public string Validation
		{
			get
			{
				return "SAS Sample";
			}
		}

		public string TaskCategory
		{
			get
			{
				return Global.Category;
			}
		}

		public string IconAssembly
		{
			get
			{
				// return the full path/name of this assembly
				return System.Reflection.Assembly.GetExecutingAssembly().Location;
			}
		}

		public string Clsid
		{
			get
			{
				return "083565a5-7f3b-4d4e-a0ee-2278067f66ff";
			}
		}

		public SAS.Shared.AddIns.ShowType TaskType
		{
			get
			{
				return SAS.Shared.AddIns.ShowType.Wizard;
			}
		}

		public bool RequiresData
		{
			get
			{
				return true;
			}
		}

		public string IconName
		{
			get
			{
				// return the name of the icon within our assembly
				// including namespace qualifiers
				return "EGAddin.Contents.Contents.ico";
			}
		}

		public int MinorVersion
		{
			get
			{
				return 0;
			}
		}

		public int MajorVersion
		{
			get
			{
				return 1;
			}
		}

		public int NumericColsRequired
		{
			get
			{
				return 0;
			}
		}

		public string TaskDescription
		{
			get
			{
				return "Runs PROC CONTENTS on the input data.";
			}
		}

		public bool GeneratesSasCode
		{
			get
			{
				return true;
			}
		}

		public string FriendlyName
		{
			get
			{
				return "Data set contents";
			}
		}

		public string ProcsUsed
		{
			get
			{
				return "CONTENTS";
			}
		}

		public string WhatIsDescription
		{
			get
			{
				return "Runs PROC CONTENTS on the input data.";
			}
		}

		public string ProductsOptional
		{
			get
			{
				return "";
			}
		}

		#endregion

		#region ISASTask Members

		public string RunLog
		{
			get
			{
				return null;
			}
		}

		public string XmlState
		{
			get
			{
				return WriteXML();
			}
			set
			{
				ReadXML(value);
			}
		}

		public void Terminate()
		{
		}

		public SAS.Shared.AddIns.OutputData OutputDataInfo(int Index, out string Source, out string Label)
		{
			if (Index == 0 && (output == OutputFormat.Data || output == OutputFormat.ReportAndData))
			{
				Source = string.Format("WORK.CTS{0}",consumer.ActiveData.Member);
				Label = label;
				return SAS.Shared.AddIns.OutputData.Libref;
			}
			else
			{
				// no output data created
				Source = null;
				Label = null;
				return SAS.Shared.AddIns.OutputData.Unknown;
			}
		}

		public string SasCode
		{
			get
			{
				return BuildProcContentsCode();
			}
		}

		public bool Initialize()
		{
			return true;
		}

		public SAS.Shared.AddIns.ShowResult Show(IWin32Window Owner)
		{
			if (consumer==null || consumer.ActiveData==null)
				throw new AddInException("Cannot get to active data",ErrorType.None);

			ContentsForm dlg = new ContentsForm();
			dlg.Text = label;
			dlg.IgnoreCase = ignorecase;
			dlg.ShowDetails = details;
			dlg.OutputFormat = output;

			if (DialogResult.OK == dlg.ShowDialog(Owner))
			{
				// update the local values with dialog settings
				ignorecase = dlg.IgnoreCase;
				details = dlg.ShowDetails;
				output = dlg.OutputFormat;
				return SAS.Shared.AddIns.ShowResult.RunNow;
			}
			else 
				return SAS.Shared.AddIns.ShowResult.Canceled;
		}

		public string Label
		{
			get
			{
				return label;
			}
			set
			{
				label = value;
			}
		}

		public int OutputDataCount
		{
			get
			{
				if (output == OutputFormat.Data || output == OutputFormat.ReportAndData)
					return 1;
				else
					return 0;
			}
		}

		#endregion

		private string BuildProcContentsCode()
		{
			StringBuilder sb = new StringBuilder();
			if (consumer!=null && consumer.ActiveData!=null)
			{
				// build up the code based on the settings
				string data = string.Format("{0}.{1}",consumer.ActiveData.Library, consumer.ActiveData.Member);
				string detailsopt = details ? "\n\tDETAILS " : "\n\tNODETAILS ";
				string sortopt = ignorecase ? "\n\tORDER=IGNORECASE " : "";
				string printopt = ((output == OutputFormat.ReportAndData) || (output == OutputFormat.Report)) ? "" : "\n\tNOPRINT ";
				string outopt = ((output == OutputFormat.ReportAndData) || (output == OutputFormat.Data)) ? string.Format("\n\tOUT=WORK.CTS{0} ",consumer.ActiveData.Member) : "";
				sb.AppendFormat("PROC CONTENTS DATA={0}{1}{2}{3}{4}; \n\tRUN;",data, detailsopt, sortopt, printopt, outopt);
			}
			return sb.ToString();
		}

		#region Save and load XML state
		private string WriteXML()
		{
			StringWriter sw = new StringWriter();

			XmlTextWriter writer = new XmlTextWriter(sw);
			writer.WriteStartElement("Contents");
			writer.WriteElementString("Details", XmlConvert.ToString(details));
			writer.WriteElementString("IgnoreCase", XmlConvert.ToString(ignorecase));
			writer.WriteElementString("OutputFormat", XmlConvert.ToString((int)output));
			writer.WriteEndElement();
			writer.Close();

			return sw.ToString();
		}

		private void ReadXML(string xml)
		{
			if (xml!=null && xml.Length>0)
			{
				try
				{
					StringReader sr = new StringReader(xml);
					XmlTextReader reader = new XmlTextReader(sr);
					reader.ReadStartElement("Contents");
					details = XmlConvert.ToBoolean(reader.ReadElementString("Details"));
					ignorecase = XmlConvert.ToBoolean(reader.ReadElementString("IgnoreCase"));
					output = (OutputFormat) XmlConvert.ToInt32(reader.ReadElementString("OutputFormat"));
					reader.ReadEndElement();
					reader.Close();
				}
				catch
				{
				}
			}
		}
		#endregion

	}
}
