using System;
using System.Windows.Forms;
using System.Xml;		// for XMLTextWriter and XMLTextReader
using System.IO;		// for StreamReader and StreamWriter
using System.Text;		// for StringBuilder

using SAS.Shared.AddIns;	// for access to the Add-in interfaces

namespace EGAddin.SASProducts
{
	/// <summary>
	/// Determine what products are available from the SAS server
	/// </summary>
	public class SASProducts : ISASTask, ISASTaskDescription, ISASTaskAddIn
	{
		private ISASTaskConsumer consumer = null;
		
		public SASProducts()
		{
		}

		#region ISASTask Members

		/// <summary>
		/// No log generated in this task
		/// </summary>
		public string RunLog
		{
			get
			{
				return null;
			}
		}

		/// <summary>
		/// Used to store state.  This task doesn't need anything
		/// but the name of the current server, and this is stored by
		/// the application.
		/// </summary>
		public string XmlState
		{
			get
			{
				return null;
			}
			set
			{
			}
		}

		public void Terminate()
		{
		}

		/// <summary>
		/// Not called in this task
		/// </summary>
		/// <param name="Index"></param>
		/// <param name="Source"></param>
		/// <param name="Label"></param>
		/// <returns></returns>
		public OutputData OutputDataInfo(int Index, out string Source, out string Label)
		{
			Source = null;
			Label = null;
			return new OutputData ();
		}

		/// <summary>
		/// Assemble the SAS code to create a report
		/// </summary>
		public string SasCode
		{
			get
			{
				// pull in data step to discover products
				string ds = Global.ReadFileFromAssembly("EGAddin.SASProducts.sasproducts.sas");

				// pull in code to create a simple report
				string rep = Global.ReadFileFromAssembly("EGAddin.SASProducts.sasproductsrep.sas");

				// concatenate the two and return as complete program
				return ds + rep;
			}
		}

		public bool Initialize()
		{
			return true;
		}

		/// <summary>
		/// Show the SAS Products form
		/// </summary>
		/// <param name="Owner">Application window</param>
		/// <returns>Whether to run the task as a report</returns>
		public ShowResult Show(IWin32Window Owner)
		{	
			// construct the form
			SASProductsForm dlg = new SASProductsForm();
			dlg.Consumer = consumer;

			// show it
			dlg.ShowDialog(Owner);

			// if we're to create a report, return RunNow
			if (dlg.CreateReport)
			{
				// tell the application what server to use
				consumer.AssignedServer = dlg.CurrentServer;
				return ShowResult.RunNow;
			}
			// else return Canceled
			else
				return ShowResult.Canceled;
		}

		/// <summary>
		/// No special label handling
		/// </summary>
		public string Label
		{
			get
			{
				return null;
			}
			set
			{
			}
		}

		/// <summary>
		/// No data created that we need to account for
		/// </summary>
		public int OutputDataCount
		{
			get
			{
				return 0;
			}
		}


		#endregion

		#region ISASTaskDescription Members

		public string ProductsRequired
		{
			get
			{
				return "BASE";
			}
		}

		public bool StandardCategory
		{
			get
			{
				return false;
			}
		}

		public bool GeneratesListOutput
		{
			get
			{
				return false;
			}
		}

		public string TaskName
		{
			get
			{
				return "SAS Product Checker";
			}
		}

		public string Validation
		{
			get
			{
				return "SAS-supplied example";
			}
		}

		public string TaskCategory
		{
			get
			{
				return Global.Category;
			}
		}

		public string IconAssembly
		{
			get
			{
				// return the full path/name of this assembly
				return System.Reflection.Assembly.GetExecutingAssembly().Location;
			}
		}

		public string Clsid
		{
			get
			{
				return "50c9ee17-6b05-43ca-9848-47b8a8d143c4";
			}
		}

		public ShowType TaskType
		{
			get
			{
				return ShowType.Wizard;
			}
		}

		public bool RequiresData
		{
			get
			{
				return false;
			}
		}

		public string IconName
		{
			get
			{
				return "EGAddin.SASProducts.products.ico";
			}
		}

		public int MinorVersion
		{
			get
			{
				return 0;
			}
		}

		public int MajorVersion
		{
			get
			{
				return 1;
			}
		}

		public int NumericColsRequired
		{
			get
			{
				return 0;
			}
		}

		public string TaskDescription
		{
			get
			{
				return "Show what SAS products are licensed and installed on a server.";
			}
		}

		public bool GeneratesSasCode
		{
			get
			{
				return true;
			}
		}

		public string FriendlyName
		{
			get
			{
				return "SAS Products Checker";
			}
		}

		public string ProcsUsed
		{
			get
			{
				return "DATA step";
			}
		}

		public string WhatIsDescription
		{
			get
			{
				return "Show what SAS products are licensed and installed on a server.";
			}
		}

		public string ProductsOptional
		{
			get
			{
				return "";
			}
		}

		#endregion

		#region ISASTaskAddIn Members

		public bool VisibleInManager
		{
			get
			{
				return true;
			}
		}

		public string AddInName
		{
			get
			{
				return "SAS Products Checker";
			}
		}

		public void Disconnect()
		{
			consumer = null;
		}

		public int Languages(out string[] Items)
		{
			Items = new string[] { "en-US" };
			return 1;
		}

		public string AddInDescription
		{
			get
			{				
				return "Reports on which SAS products are licensed and installed on the selected server.";
			}
		}

		public bool Connect(ISASTaskConsumer Consumer)
		{
			consumer = Consumer;
			return true;
		}

		public string Language
		{
			set
			{
			}
		}

		#endregion

	}
}
