using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;
using System.Data.OleDb;


using SAS.Shared.AddIns;	// for access to the Add-in interfaces

namespace EGAddin.SASProducts
{
	/// <summary>
	/// Summary description for SASProductsForm.
	/// </summary>
	public class SASProductsForm : System.Windows.Forms.Form
	{
		#region Private members
		/// <summary>
		/// Required designer variable.
		/// </summary>
		private System.ComponentModel.Container components = null;
		private System.Windows.Forms.ComboBox cmbServers;
		private System.Windows.Forms.Label label1;
		private System.Windows.Forms.Button btnClose;
		private ISASTaskConsumer consumer = null;
		private SASListView lvProducts;
		private System.Windows.Forms.Button btnCheck;
		private System.Windows.Forms.Label lblDesc;
		private string _currServer = "";
		private ImageList imgList = new ImageList();
		private System.Windows.Forms.StatusBar sbar;
		private System.Windows.Forms.Button btnReport;
		private string _sasProgram = "";
		private bool _createReport = false;

		// enum for use with image list
		enum ProdIcons : int
		{
			Licensed = 0,
			NotLicensed = 1
		}
		#endregion

		#region Properties
		// the ISASTaskConsumer from the host application
		public ISASTaskConsumer Consumer
		{
			set
			{
				consumer = value;
			}
		}

		public bool CreateReport
		{
			get { return _createReport; }
		}

		public string CurrentServer
		{
			get { return _currServer; }
		}
		#endregion

		#region Constructor and cleanup
		/// <summary>
		/// Constructor
		/// </summary>
		public SASProductsForm()
		{
			//
			// Required for Windows Form Designer support
			//
			InitializeComponent();

			// read in our SAS program
			_sasProgram = Global.ReadFileFromAssembly("EGAddin.SASProducts.sasproducts.sas");

			// set up image list for use in the form
			imgList.Images.Add(Global.GetImageFromResources("SASProducts.licensed.ico", new Size(16,16)));
			imgList.Images.Add(Global.GetImageFromResources("SASProducts.notlicensed.ico", new Size(16,16)));
			lvProducts.SmallImageList = imgList;
			lvProducts.LargeImageList = imgList;
		}

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if(components != null)
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}
		#endregion

		#region Form setup and content
		/// <summary>
		/// Override of OnLoad to populate form with some information
		/// </summary>
		/// <param name="e"></param>
		protected override void OnLoad(EventArgs e)
		{
			base.OnLoad (e);

			if (consumer == null)
				throw new System.Exception("Must set ISASTaskConsumer as Consumer property!");
			
			// get the list of available servers from the application
			string[] servers;
			consumer.Servers(out servers);

			if (servers.Length == 0)
			{
				servers = new string[1] { consumer.AssignedServer };
			}

			// populate the servers combobox
			cmbServers.Items.AddRange(servers);			
			if (consumer.AssignedServer.Length>0)
				cmbServers.SelectedItem = consumer.AssignedServer;

			_currServer = cmbServers.SelectedItem.ToString();

			ColumnHeader ch = new ColumnHeader();
			ch.Text = "Product Name";
			ch.Width = lvProducts.Width / 2;
			lvProducts.Columns.Add(ch);
			ch = new ColumnHeader();
			ch.Text = "Licensed?";
			ch.Width = lvProducts.Width / 4;
			lvProducts.Columns.Add(ch);
			ch = new ColumnHeader();
			ch.Text = "Installed?";
			ch.Width = lvProducts.Width / 4;
			lvProducts.Columns.Add(ch);
		}

		// fill in the list view with the interpretation of the WORK.SASPRODS data set
		// that we created with the code that was submitted.
		private void PopulateProductList()
		{
			Cursor c = Cursor.Current;
			Cursor.Current = Cursors.WaitCursor;

			SAS.Workspace ws = null;
			try
			{
				ws = consumer.Workspace(_currServer) as SAS.Workspace;		
			}
			catch (Exception ex)
			{
				throw new System.Exception("ISASTaskConsumer.Workspace is not usable!",ex);
			}

			if (_currServer.Length>0 && ws!=null)
			{

				// use the SAS IOM OLEDB provider to read data from the SAS workspace
				ADODB.Recordset adorecordset = new ADODB.RecordsetClass();
				ADODB.Connection adoconnect = new ADODB.ConnectionClass();

				try
				{
					adoconnect.Open("Provider=sas.iomprovider.1; SAS Workspace ID=" + ws.UniqueIdentifier, "", "", 0);
					
					// query the data set created by the SAS code that was submitted
					string selectclause = "select prodname, licensed, installed from work.sasprods";
					adorecordset.Open( selectclause, adoconnect, ADODB.CursorTypeEnum.adOpenForwardOnly, ADODB.LockTypeEnum.adLockReadOnly, (int) ADODB.CommandTypeEnum.adCmdText);
					
					while (!adorecordset.EOF)
					{	
						ListViewItem lvi = new ListViewItem(adorecordset.Fields["prodname"].Value.ToString());
						string licensed = (adorecordset.Fields["licensed"].Value.ToString().Trim() == "1") ? "Yes" : "No";
						string installed = adorecordset.Fields["installed"].Value.ToString().Trim();
						if (installed != "<unknown>")
							installed = (installed.Length>0) ? "Yes" : "No";
						lvi.SubItems.Add(licensed);
						lvi.SubItems.Add(installed);
						if (licensed == "Yes" && installed=="No")
							lvi.ForeColor = Color.OliveDrab;
						else if (licensed == "No" && installed=="No")
							lvi.ForeColor = SystemColors.GrayText;
						lvi.ImageIndex = (licensed == "Yes") ? (int)ProdIcons.Licensed : (int)ProdIcons.NotLicensed;
						lvProducts.Items.Add(lvi);
						adorecordset.MoveNext();
					}

				}
				catch {}
				finally
				{
					adoconnect.Close();
				}

			}

			Cursor.Current = c;
			UpdateControls(true);
			sbar.Text = string.Format("Analysis of {0} complete!",_currServer);
		}

		#endregion

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			System.Resources.ResourceManager resources = new System.Resources.ResourceManager(typeof(SASProductsForm));
			this.cmbServers = new System.Windows.Forms.ComboBox();
			this.label1 = new System.Windows.Forms.Label();
			this.lvProducts = new SASListView();
			this.btnClose = new System.Windows.Forms.Button();
			this.btnCheck = new System.Windows.Forms.Button();
			this.lblDesc = new System.Windows.Forms.Label();
			this.sbar = new System.Windows.Forms.StatusBar();
			this.btnReport = new System.Windows.Forms.Button();
			this.SuspendLayout();
			// 
			// cmbServers
			// 
			this.cmbServers.DropDownStyle = System.Windows.Forms.ComboBoxStyle.DropDownList;
			this.cmbServers.Location = new System.Drawing.Point(12, 68);
			this.cmbServers.Name = "cmbServers";
			this.cmbServers.Size = new System.Drawing.Size(246, 21);
			this.cmbServers.TabIndex = 3;
			this.cmbServers.SelectedIndexChanged += new System.EventHandler(this.cmbServers_SelectedIndexChanged);
			// 
			// label1
			// 
			this.label1.Location = new System.Drawing.Point(12, 49);
			this.label1.Name = "label1";
			this.label1.Size = new System.Drawing.Size(249, 18);
			this.label1.TabIndex = 2;
			this.label1.Text = "Select the SAS server to examine:";
			// 
			// lvProducts
			// 
			this.lvProducts.Anchor = ((System.Windows.Forms.AnchorStyles)((((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Bottom) 
				| System.Windows.Forms.AnchorStyles.Left) 
				| System.Windows.Forms.AnchorStyles.Right)));
			this.lvProducts.FullRowSelect = true;
			this.lvProducts.GridLines = true;
			this.lvProducts.Location = new System.Drawing.Point(5, 100);
			this.lvProducts.MultiSelect = false;
			this.lvProducts.Name = "lvProducts";
			this.lvProducts.Size = new System.Drawing.Size(425, 233);
			this.lvProducts.TabIndex = 4;
			this.lvProducts.View = System.Windows.Forms.View.Details;
			// 
			// btnClose
			// 
			this.btnClose.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Bottom | System.Windows.Forms.AnchorStyles.Right)));
			this.btnClose.FlatStyle = System.Windows.Forms.FlatStyle.System;
			this.btnClose.Location = new System.Drawing.Point(356, 341);
			this.btnClose.Name = "btnClose";
			this.btnClose.TabIndex = 5;
			this.btnClose.Text = "Close";
			this.btnClose.Click += new System.EventHandler(this.btnClose_Click);
			// 
			// btnCheck
			// 
			this.btnCheck.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
			this.btnCheck.FlatStyle = System.Windows.Forms.FlatStyle.System;
			this.btnCheck.Location = new System.Drawing.Point(274, 68);
			this.btnCheck.Name = "btnCheck";
			this.btnCheck.Size = new System.Drawing.Size(154, 23);
			this.btnCheck.TabIndex = 6;
			this.btnCheck.Text = "Show SAS products";
			this.btnCheck.Click += new System.EventHandler(this.btnCheck_Click);
			// 
			// lblDesc
			// 
			this.lblDesc.Anchor = ((System.Windows.Forms.AnchorStyles)(((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Left) 
				| System.Windows.Forms.AnchorStyles.Right)));
			this.lblDesc.Location = new System.Drawing.Point(11, 10);
			this.lblDesc.Name = "lblDesc";
			this.lblDesc.Size = new System.Drawing.Size(414, 34);
			this.lblDesc.TabIndex = 7;
			this.lblDesc.Text = "Use this task to determine which SAS products are licensed and installed on your " +
				"SAS server.";
			// 
			// sbar
			// 
			this.sbar.Location = new System.Drawing.Point(0, 369);
			this.sbar.Name = "sbar";
			this.sbar.Size = new System.Drawing.Size(437, 22);
			this.sbar.TabIndex = 8;
			// 
			// btnReport
			// 
			this.btnReport.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Bottom | System.Windows.Forms.AnchorStyles.Right)));
			this.btnReport.FlatStyle = System.Windows.Forms.FlatStyle.System;
			this.btnReport.Location = new System.Drawing.Point(225, 341);
			this.btnReport.Name = "btnReport";
			this.btnReport.Size = new System.Drawing.Size(119, 23);
			this.btnReport.TabIndex = 9;
			this.btnReport.Text = "Create Report";
			this.btnReport.Click += new System.EventHandler(this.btnReport_Click);
			// 
			// SASProductsForm
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.ClientSize = new System.Drawing.Size(437, 391);
			this.Controls.Add(this.btnReport);
			this.Controls.Add(this.sbar);
			this.Controls.Add(this.lblDesc);
			this.Controls.Add(this.btnCheck);
			this.Controls.Add(this.btnClose);
			this.Controls.Add(this.lvProducts);
			this.Controls.Add(this.cmbServers);
			this.Controls.Add(this.label1);
			this.Icon = ((System.Drawing.Icon)(resources.GetObject("$this.Icon")));
			this.MaximizeBox = false;
			this.MinimizeBox = false;
			this.MinimumSize = new System.Drawing.Size(445, 290);
			this.Name = "SASProductsForm";
			this.ShowInTaskbar = false;
			this.StartPosition = System.Windows.Forms.FormStartPosition.CenterParent;
			this.Text = "SAS Products Checker";
			this.ResumeLayout(false);

		}
		#endregion

		#region Event handlers
		private void btnClose_Click(object sender, System.EventArgs e)
		{
			this.Close();
		}

		private void UpdateControls(bool bEnable)
		{
			btnCheck.Enabled = bEnable;
			cmbServers.Enabled = bEnable;
		}

		private void btnCheck_Click(object sender, System.EventArgs e)
		{
			sbar.Text = string.Format("Analyzing products on {0}...",_currServer);
			UpdateControls(false);

			// submit the SAS program to determine licensed/installed products
			ISASTaskSubmit sub = consumer.Submit as ISASTaskSubmit;
			if (sub!=null)
			{
				Cursor c = Cursor.Current;
				Cursor.Current = Cursors.WaitCursor;

				lvProducts.Items.Clear();
				lvProducts.Message =  string.Format("Analyzing products on {0}...",_currServer);

				sub.SubmitCode(_sasProgram, _currServer, false);

				Cursor.Current = c;

				PopulateProductList();
			}
			else 
				MessageBox.Show("This application does not support submitting code from a task.","Not supported");

		}

		private void cmbServers_SelectedIndexChanged(object sender, System.EventArgs e)
		{
			_currServer = cmbServers.SelectedItem.ToString();
		}

		private void btnReport_Click(object sender, System.EventArgs e)
		{
			_createReport = true;
			this.Close();
		}

		#endregion
	}
}
