// ---------------------------------------------------------------
// Copyright 2004, SAS Institute Inc.
// ---------------------------------------------------------------
using System;
using System.Windows.Forms;
using System.Xml;		// for XMLTextWriter and XMLTextReader
using System.IO;		// for StreamReader and StreamWriter
using System.Text;		// for StringBuilder

using SAS.Shared.AddIns;	// for access to the Add-in interfaces


namespace EGAddin.SysCommand
{
	/// <summary>
	/// Processes system commands as if in a Windows batch file
	/// </summary>
	public class SysCommand : ISASTask, ISASTaskAddIn, ISASTaskDescription
	{
		public SysCommand()
		{
		}
	
		private string log = "";
		private string cmds = "REM Add commands to execute";

		#region ISASTaskAddIn Members

		public bool VisibleInManager
		{
			get
			{
				return true;
			}
		}

		public string AddInName
		{
			get
			{
				return "Windows System Command";
			}
		}

		public void Disconnect()
		{
		}

		public int Languages(out string[] Items)
		{
			Items = new string[] { "en-US" };
			return 1;
		}

		public string AddInDescription
		{
			get
			{
				return "Processes Windows system commands as if in a batch file.";
			}
		}

		public bool Connect(ISASTaskConsumer Consumer)
		{
			return true;
		}

		public string Language
		{
			set
			{
				// no implementation
			}
		}

		#endregion

		#region ISASTaskDescription Members

		public string ProductsRequired
		{
			get
			{
				return "";
			}
		}

		public bool StandardCategory
		{
			get
			{
				return false;
			}
		}

		public bool GeneratesListOutput
		{
			get
			{
				return false;
			}
		}

		public string TaskName
		{
			get
			{
				return "System Command";
			}
		}

		public string Validation
		{
			get
			{
				return "SAS Sample";
			}
		}

		public string TaskCategory
		{
			get
			{
				return Global.Category;
			}
		}

		public string IconAssembly
		{
			get
			{
				// return the full path/name of this assembly
				return System.Reflection.Assembly.GetExecutingAssembly().Location;
			}
		}

		public string Clsid
		{
			get
			{
				return "13a5a715-e3ca-4b41-a944-df5dde0e5ffc";
			}
		}

		public ShowType TaskType
		{
			get
			{
				return ShowType.Wizard;
			}
		}

		public bool RequiresData
		{
			get
			{
				return false;
			}
		}

		public string IconName
		{
			get
			{
				return "EGAddin.SysCommand.SysCommand.ico";
			}
		}

		public int MinorVersion
		{
			get
			{
				return 0;
			}
		}

		public int MajorVersion
		{
			get
			{
				return 1;
			}
		}

		public int NumericColsRequired
		{
			get
			{
				return 0;
			}
		}

		public string TaskDescription
		{
			get
			{
				return "Accepts a list of system commands, then builds a batch file and executes them.  The log is saved as part of the project.";
			}
		}

		public bool GeneratesSasCode
		{
			get
			{
				return false;
			}
		}

		public string FriendlyName
		{
			get
			{
				return "System Command";
			}
		}

		public string ProcsUsed
		{
			get
			{
				return "";
			}
		}

		public string WhatIsDescription
		{
			get
			{
				return "Accepts a list of system commands, then builds a batch file and executes them.  The log is saved as part of the project.";			
			}
		}

		public string ProductsOptional
		{
			get
			{
				return "";
			}
		}

		#endregion

		#region ISASTask Members

		// the act of retrieving the log actually will cause this task to run
		public string RunLog
		{
			get
			{
				// execute the commands stored 
				log = ExecuteCommands();
				return log;
			}
		}

		// save and restore the state (commands)
		public string XmlState
		{
			get
			{
				using(StringWriter sw = new StringWriter())
				{
					XmlTextWriter writer = new XmlTextWriter(sw);
					writer.WriteStartElement("SysCommands");
					writer.WriteElementString("Commands", cmds);
					writer.WriteEndElement();
					writer.Close();
					return sw.ToString();
				}
			}
			set
			{
				if (value!=null && value.Length>0)
				{
					try
					{
						using (StringReader sr = new StringReader(value))
						{
							XmlTextReader reader = new XmlTextReader(sr);
							reader.ReadStartElement("SysCommands");
							cmds = reader.ReadElementString("Commands");
							reader.ReadEndElement();
							reader.Close();
						}
					}
					catch
					{
					}
				}			
			}
		}

		// nothing special needed at init or terminate time
		public bool Initialize()
		{
			return true;
		}

		public void Terminate()
		{
		}

		// this task does not generate output data
		public int OutputDataCount
		{
			get
			{
				return 0;
			}
		}

		public OutputData OutputDataInfo(int Index, out string Source, out string Label)
		{
			Source = null;
			Label = null;
			return OutputData.Unknown;
		}

		// this task returns no SAS code
		public string SasCode
		{
			get
			{
				return null;
			}
		}

		public ShowResult Show(IWin32Window Owner)
		{
			SysCommandForm dlg = new SysCommandForm();
			dlg.Cmds = cmds;
			dlg.Text = label;
			if (dlg.ShowDialog(Owner) == DialogResult.OK)
			{
				cmds = dlg.Cmds;
				return ShowResult.RunNow;
			}
			else
				return ShowResult.Canceled;
		}

		// respect the label that was set on the task, for display in the task
		// title bar in the Show method
		private string label = "Windows System Commands";
		public string Label
		{
			get
			{
				return label;
			}
			set
			{
				label = value;
			}
		}

		#endregion

		#region Execute commands in a batch file
		public string ExecuteCommands()
		{
			string stdout;
			try
			{
				// create a temp batch file to run the commands
				string fn = Path.GetTempFileName();
				File.Move(fn, fn+".cmd");
				fn += ".cmd";

				using (StreamWriter sw = new StreamWriter(fn))
				{
					sw.Write(cmds);
				}

				// launch a process to run the batch file
				// record the stdout so we can report in the log for the task
				System.Diagnostics.ProcessStartInfo psi = new System.Diagnostics.ProcessStartInfo();
				psi.FileName = fn;
				psi.RedirectStandardOutput = true;
				psi.UseShellExecute = false;
				psi.CreateNoWindow = true;

				System.Diagnostics.Process p = System.Diagnostics.Process.Start(psi);
				stdout = p.StandardOutput.ReadToEnd();

				p.WaitForExit();

				// cleanup the temp file
				File.Delete(fn);

				// return the stdout
				return stdout;
			}
			catch (Exception ex)
			{
				return "ERROR: Could not write/run batch file.\n" + ex.ToString();
			}
			
		}
		#endregion
	}
}
